
#include <math.h>

#include "Boid.h"

#include "Environment.h"
#include "Sensor.h"


#define DEFAULT_CENTER_BIAS 0.14
#define DEFAULT_AVG_VEL     0.04
#define ACCEL_BIAS			2.00
#define AVOID_BIAS			2.20





const float DEG2RAD = 3.14159/180;



Boider::Boider()
{


	initialized = false;
}

int Boider::cull()
{

	if(!initialized)
		initialize();

	draw();
	
	
	return 1;
}

void Boider::initialize()
{



	boidGeometry = glGenLists(1);

	glNewList(boidGeometry, GL_COMPILE);
	
	
	
	
	float bubbleSize = 20.0;

	
	glBegin(GL_POLYGON);
	
	glTexCoord2f(0.0, 0.0);
	glVertex2f(-bubbleSize, -bubbleSize);
	glTexCoord2f(1.0, 0.0);
	glVertex2f(bubbleSize, -bubbleSize);
	glTexCoord2f(1.0, 1.0);
	glVertex2f(bubbleSize, bubbleSize);
	glTexCoord2f(0.0, 1.0);
	glVertex2f(-bubbleSize, bubbleSize);
	
	glEnd();
	
	
	glEndList();
	
	initialized = true;
	
}


void Boider::draw()
{

	if(tex && !tex->isLoaded())
	{
		
		tex->load();
	
	}
	
	Boid* boid = reinterpret_cast<Boid*>(getParent());

	
	
	
	glColor4f(1.0, 1.0, 1.0, 0.5);
	
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	
	
	
	if(tex)
	{
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, *(tex->getId()));
			
	}
	
	glCallList(boidGeometry);


	if(tex)
	{
		glBindTexture(GL_TEXTURE_2D, 0);
		glDisable(GL_TEXTURE_2D);
	}

	glDisable(GL_BLEND);

}

RTTI_NODE_TYPE Boider::getType() const
{

	return RTTI_USER0;
}


Boid::Boid()
{

}

Boid::Boid(Texture *blt)
{
	
	bubbleTex = blt;
	
	boider = new Boider();
	boider->addTexture(bubbleTex);
	attachChild(boider);
	
	
	
	initialized = false;
	
	
	pos.set(glTransform()[12], glTransform()[13], glTransform()[14]);
	vel.set(float(rrand(100)-50)/ 10.0, float(rrand(100)-50)/ 10.0, 0.0);
	orientation.set(0.0, 1.0, 0.0);
	

	searchRadius = 0.10;
	
	mode = NOMODE;
	
	
	outOfBound = false;
	
	myBucket = (Bucket*) 0;
	
}

void Boid::initialize()
{



	initialized = true;
}

int Boid::cull()
{

	if(!initialized)
		initialize();
		
		
	float FRUSTUM[6][4];
	Frustum::get_frustum(FRUSTUM);
	
	float p[3];
	p[0] = 0.0;
	p[1] = 0.0;
	p[2] = 0.0f;
		
	if(Frustum::test_point(FRUSTUM, p) != -1)
		draw();
	else
		return -1;
		
	return 1;

}

int Boid::checkBucket(Bucket* a, Bucket* b)
{

	if(outOfBound)
		return 4;


	if(!b)
		return 3;

	if(!a && b)
		return 2;


	if(a && b) 
	{
		if(a->x() == b->x() && a->y() == b->y())
			return 1;
		else 
			return -1;
	}
	
	return 0;
}



void Boid::setId(int id)
{
	boidId = id;

}

int Boid::getId()
{

	return boidId;
	
}


void Boid::doupdate()
{
	
	
	Environment *env = reinterpret_cast<Environment*>(getParent());
	
	
	pos.x = glTransform()[12];
	pos.y = glTransform()[13];
	pos.z = glTransform()[14];
	
	

	
	Bucket* bucket;
	bucket = env->getBucketIndex()->query(pos.x * 10.0f, pos.y * 10.0f);

	int result = checkBucket(myBucket, bucket);
	switch(result)
	{
		case 2:	
		
			myBucket = bucket;
			myBucket->add(this);
		
			break;
			
		case 1:
		
			break;
		
		case -1:
		
			myBucket->remove(this);
			
			myBucket = bucket;
			myBucket->add(this);
			break;
			
		case 0:
		
			break;
		
		case 3:
			myBucket->remove(this);
			myBucket = (Bucket*) 0;
			
			outOfBound = true;
			break;
		case 4:
			break;
	}
	
	
	// query for neighboring boids
	
	if(myBucket)
	{
	
				
		std::vector<Bucket*> bucketlist;
		env->getBucketIndex()->query(&bucketlist, 
			((pos.x * 10.0f) - 1.0f), ((pos.y * 10.0f) - 1.0f), 
			((pos.x * 10.0f) + 1.5f), ((pos.y * 10.0f) + 1.5f));
		
	
		//printf("number of buckets: %i\n", bucketlist.size());
		
		std::vector<Boid*> flockmates;
		std::vector<Boid*> allneighbors;
		
		for(int i = 0; i < bucketlist.size(); i++)
		{
		
			for(int j = 0; j < bucketlist[i]->getBoids()->size(); j++)
			{
				if((*bucketlist[i]->getBoids())[j] != (Node*) 0)
				{
				
					Boid *b = 
						reinterpret_cast<Boid*>((*bucketlist[i]->getBoids())[j]);
					
					
					if(!b || b->getId() == boidId) continue;
					
			
					
					float distance = Vec3::EuclDistance(b->pos, pos);
					
					
					
					if(distance <= searchRadius)
					{
						
						
						flockmates.push_back(b);	
							
						allneighbors.push_back(b);
							
					}
				}
			
			}
		
		}
	
	
	
		
		
		
		
		move(&allneighbors, &flockmates);			
	}
	else
	{
		
		//printf("boid lost the bucket.\n");
		//glTransform()[12] = 0.500;
		//glTransform()[13] = 0.500;
	
		die();
		
		//printf("Died!\n");
	
		return;
	}
}




Boid* Boid::getClosest(std::vector<Boid*>* neighbors)
{

	int numboids = neighbors->size();
	
	Boid *closest = (Boid*) 0;
	
	float CLDIST = 100000;
	
	// find closest neighbor
	for(int i=0; i < numboids; i++) 
	{
		float dist = Vec3::EuclDistance((*neighbors)[i]->pos, pos);
		
		if(dist < CLDIST)
		{
		
			CLDIST = dist;
			
			closest = (*neighbors)[i];
			
			
		}
		
    }
    
    return closest;

}

void Boid::getAvoidVector(Vec3& avoid, Boid* boid, BOID_COLLISION_MODE mode)
{

	Vec3 separation;
	
	
	
	float distance = Vec3::EuclDistance(pos, boid->pos);
	
	separation = pos - boid->pos;
	
	int initial;
	int secondary;

	switch(mode)
	{
	
		case HARDBOUNCE:
		
			initial = 16;
			secondary = 120;
			break;
			
		case BOUNCE:
		
			initial = 8;
			secondary = 40;
			break;
		
		case SOFT:
		
			initial = 4;
			secondary = 4;
			break;
			
	
	}
	
	
	float cruiseDistance = 0.03;
	
	
	
	
	if(distance < cruiseDistance / 2.0f)
	{
	
		avoid.x = float(rrand(secondary) + (secondary/2)) * separation.x;
		avoid.y = float(rrand(secondary) + (secondary/2)) * separation.y;
		avoid.z = float(rrand(secondary) + (secondary/2)) * separation.z;
		
	}
	else if(distance < cruiseDistance)
	{
		
		avoid.x = float(rrand(initial) + (initial/2)) * separation.x;
		avoid.y = float(rrand(initial) + (initial/2)) * separation.y;
		avoid.z = float(rrand(initial) + (initial/2)) * separation.z;
		
	
	}
	

}

void Boid::updateFlockingCenter(Vec3 *vec)
{

	flockCenter.x = vec->x;
	flockCenter.y = vec->y;
	flockCenter.z = vec->z;
}


void Boid::doBoidCollision(std::vector<Boid*>* all)
{

	Boid* closest = getClosest(all);

	if(closest)
	{
		Vec3 avoid;
		
		
		getAvoidVector(avoid, closest, BOUNCE);
		
		
		
		if(avoid.isnonzero())
		{
			avoid = avoid * AVOID_BIAS;
			
			vel = vel + avoid;
		}
	}


}


Node* Boid::getClosestSensor()
{

	
	
	Node* closest = (Node*) 0;
	
	
	float CLDIST = 100000;
	
	for(int i = 0; i < sensorList->size(); i++)
	{
	
		Sensor* thisSensor = reinterpret_cast<Sensor*>((*sensorList)[i]);
			
		float distance = Vec3::EuclDistance(thisSensor->pos, pos);
		//printf("%i, distance = %f\n", thisSensor->sensorId, distance);
		if(distance < CLDIST)
		{
		
			CLDIST = distance;
			
			closest = (*sensorList)[i];
		
		}
		
		
	}

	

	return closest;

}


void Boid::moveToClosestSensor()
{

	Sensor* closest = reinterpret_cast<Sensor*>(getClosestSensor());
	
	
	if(closest)
	{
	
		//printf("sensor id: %i\n", closest->sensorId);
		
		float distance = Vec3::EuclDistance(closest->pos, pos);
		
		if(distance < 0.25)
		{
		
			Vec3 ray;
		
			if(type == BOID_BUBBLE && distance < 0.1)
			{
			
				ray = pos - closest->pos;
				ray = ray * 25;
				
				//printf("bubble runs away\n");
			}
			else
			{
				ray = closest->pos - pos;
				
				ray = ray * 2;
			}	
			
			vel = vel + ray;
		}
		
	}
	else
		printf("sensor not found\n");
	

}

void Boid::move(std::vector<Boid*>* all, std::vector<Boid*>* flockmates)
{

	
	//printf("Moving %i\n", boidId);
	//printf("all: %i flcokmates: %i\n",
	//all->size(), flockmates->size());
	

	
	Boid* closest = (Boid *) 0;
	Boid* closestFlockMate = (Boid* ) 0;
	
	
	int numboids = flockmates->size();

	Vec3 center, center_bias;
   	Vec3 avgvelocity, avgvel_bias;
   	
	Vec3 accel;
	Vec3 oldvel;
	

	oldvel.set(vel.x, vel.y, vel.z);

	// do this for flockmates
	if(numboids != 0)
	{
		
	
		
		for(int i=0; i < numboids; i++) 
		{
		
			center = center + (*flockmates)[i]->pos;
			avgvelocity = avgvelocity + (*flockmates)[i]->vel;
	    }
	
	
		
	
		center = center / numboids;
		updateFlockingCenter(&center);
		
		avgvelocity = avgvelocity / numboids;
		
		
		center_bias = center - pos;
		avgvel_bias = avgvelocity - vel;
		
		// accelerate to catch up with the flock
		if(Vec3::EuclDistance(center, pos) >= 0.125)
		{
			avgvel_bias = avgvel_bias * ACCEL_BIAS;
			
		}
	
		
		center_bias = center_bias * DEFAULT_CENTER_BIAS;
		avgvel_bias = avgvel_bias * DEFAULT_AVG_VEL;
		
		
		closest = getClosest(all);
		closestFlockMate = getClosest(flockmates);
		
		
		
		
			
		vel = vel + center_bias;
		vel = vel + avgvel_bias;
		
	}
	
	
	
	
	/* bound world */
	if(pos.x + vel.x*0.01<= 0.05) {
	   vel.x +=0.5;
	}
	if(pos.x + vel.x*0.01>= 0.95) {
	   vel.x -=0.5;
	}
	if(pos.y + vel.y*0.01<= 0.05) {
	   vel.y += 0.5;
	}
	
	if(type != BOID_BUBBLE)
	{
		if(pos.y + vel.y*0.01>= 0.95) {
	   	vel.y -=0.5;
		}
	}
	
	

	
	doBoidCollision(all);
	moveToClosestSensor();
	
	
	if(type == BOID_BUBBLE)
	{
		
		Vec3 up(0.5, 1.5, 0);
		Vec3 ray = up - pos;
		ray = ray * 0.85;
		vel = vel + ray;
	}
	
	
	accel.set(vel.x - oldvel.x, vel.y - oldvel.y,
										vel.z - oldvel.z);
	

	
	// orientation rule
	
	Vec3 original(orientation.x, orientation.y, orientation.z);	
	Vec3 target(vel.x, vel.y, vel.z);
	
	original.normalize();
	target.normalize();
	
	float angle2rotate = Vec3::AngleBetween(original, target);
	

	
	Vec3 T = target - original;
	
	Vec3 N = Vec3::CrossProduct(T, target);
	if(ToDegree(angle2rotate) >= -180 && ToDegree(angle2rotate) <= 180)
	{
	
		if(N.z <= 0)
			boider->rotate(0, 0, 1, ToDegree(angle2rotate));
		else
			boider->rotate(0, 0, 1, -ToDegree(angle2rotate));
	}
	
	orientation.x = vel.x;
	orientation.y = vel.y;
	orientation.z = vel.z;
	
	
	vel.limit(MAX_BOID_VEL);
	
	
	translate(vel.x, vel.y, vel.z);
	
	
	
}




void Boid::draw()
{

	if(type != BOID_BUBBLE) return;

	// TODO
        float speed  = vel.length();
        float radius = 20.0f;
        radius = radius * cos(counter*DEG2RAD*speed/10.0f);

        float box_w  = 3.0f;

        // middle point
        /*
        glColor3f(1.0f, 0.0f, 0.0f);
        glPushMatrix();
        {
            glScalef(box_w, box_w, 1.0f);
            glBegin(GL_POLYGON);
            {
                glVertex2f( 1.0f, -1.0f);
                glVertex2f(-1.0f, -1.0f);
                glVertex2f(-1.0f,  1.0f);
                glVertex2f( 1.0f,  1.0f);
            }
            glEnd();
        }
        glPopMatrix();
        */

        // shape
        
        glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	


        glColor4f(0.8f, 0.8f, 1.0f, 0.5f);
        glBegin(GL_LINE_LOOP);
        {
            for(int i=0; i<360; i++)
            {
                float degInRad = i*DEG2RAD;
                float x_pos = cos(degInRad) * radius; 
                float y_pos = sin(degInRad) * radius;

                Vec3 pos(x_pos, y_pos, 0.0);
                float scale = Vec3::DotProduct(vel, pos);
                scale = scale / (vel.length()*pos.length());

                glVertex2f( scale*x_pos, scale*y_pos );
            }
        }
        glEnd();

	glDisable(GL_BLEND);

        counter++;
        counter = counter % 360 + 1;


	
}


RTTI_NODE_TYPE Boid::getType() const
{

	return RTTI_PARTICLE;


}

