
#ifndef Boid_H
#define Boid_H



#include "../../src-core/Node.h"
#include "../../src-core/Shapes.h"




#define MAX_BOID_VEL 7


class Bucket;

enum BOID_COLLISION_MODE
{
	HARDBOUNCE,
	BOUNCE,
	SOFT
};

enum BOID_MODE
{
	NOMODE,
	MOVE_TO_SENSOR
	
	
};


enum BOID_TYPE
{
	BOID_FISH,
	BOID_BUBBLE
};




class Boider: public Node
{
public:
	Boider();

	virtual int cull();
	virtual void draw();
	virtual RTTI_NODE_TYPE getType() const;
	
	
	virtual void doupdate() { }
	
	virtual void drawBBox() { }
	virtual void userEvent(UserEvent*) { }
	virtual void addTexture(Texture* t) { tex = t;}
	virtual Texture* getTexture() { }
	virtual void computeBBox() { }
	virtual BBox* getBBox() { }
	virtual void addGeometry(Geometry*) { }
	virtual Geometry* getGeometry() { }
	virtual void print() { }
	
	
	void initialize();

private:

	bool initialized;

	GLuint boidGeometry;

	Texture *tex;
};



class Boid: public Node
{

public:
	Boid();
	Boid(Texture*);	
	
	virtual int cull();
	virtual void draw();
	virtual RTTI_NODE_TYPE getType() const;
	
	
	virtual void doupdate();
	
	
	virtual void drawBBox() { }
	virtual void userEvent(UserEvent*) { }
	virtual void addTexture(Texture*) { }
	virtual Texture* getTexture() { }
	virtual void computeBBox() { }
	virtual BBox* getBBox() { }
	virtual void addGeometry(Geometry*) { }
	virtual Geometry* getGeometry() { }
	virtual void print() { }


	

	void initialize();
	
	int checkBucket(Bucket*, Bucket*);	

	void setId(int);
	int getId();
	
	
	/* boid position and velocity */
	Vec3 pos, vel;	
	Vec3 orientation;
	Vec3 flockCenter;


	std::vector<Node*>* obstacleList;
	
	std::vector<Node*>* sensorList;
	
	
	
	BOID_TYPE type;

private:

	
	BBox bbox;
	bool initialized;
	float searchRadius;


	/* boid movemet lnogic */
	void updateFlockingCenter(Vec3*);
	void doBoidCollision(std::vector<Boid*>*);
	void move(std::vector<Boid*>*, std::vector<Boid*>*);			
	Boid* getClosest(std::vector<Boid*>*);
	void getAvoidVector(Vec3&, Boid*, BOID_COLLISION_MODE);
	
	
	/* sensor movement logic */
	Node* getClosestSensor();
	void moveToClosestSensor();
	
	
	
	
	
	// unique id
	int boidId;

	Bucket* myBucket;
	Boider *boider;
	BOID_MODE mode;
	
	
	
	
	
	
	
	bool outOfBound;
	
	
	Texture *bubbleTex;
	
	
	  // FIXME
        int counter;
};



#endif