
#include "Environment.h"

#include "Boid.h"

#include "Sensor.h"
#include "Creature.h"
#include "MoCap.h"





#include <FL/Fl.H>



void dataTimer(void *pData) {
	
	if( pData != NULL ) {
	
		MoCap *mocap = reinterpret_cast<MoCap*>(pData);
		// RATKO
		float musicOffset = mocap->sound->getPlaybackOffset();
		
		// RATKO
		// Figure out the correct position of the sensor 
		// data based on the music playback position.
		mocap->step = (int) ceil(musicOffset * mocap->numSteps);
		float p = mocap->step/(float)mocap->numSteps;
		//printf("\ncurrent step: %d out of %d (percentage: %f)", mocap->step, mocap->numSteps, p);


		//mocap->step++;

	}

	
	Fl::repeat_timeout(1.0/90.0, dataTimer, pData);
}







Bucket::Bucket()
{

}

Bucket::Bucket(int c, int r)
{

	index[0] = c;
	index[1] = r;

}

int Bucket::x()
{
	return index[0];

}

int Bucket::y()
{


	return index[1];
}

std::vector<Node*>* Bucket::getBoids()
{
	return &boidList;
}


void Bucket::remove(Node* node)
{

		
	Boid *boid = reinterpret_cast<Boid*>(node);
	
	for(int i = 0; i < boidList.size(); i++)
	{
		if(!boidList[i])
			continue;
			
		Boid *thisBoid = reinterpret_cast<Boid*>(boidList[i]);
		if(boid->getId() == thisBoid->getId())
		{
			nullboids.push(i);
			boidList[i] = (Node*) 0;
		}
		
	}
}

void Bucket::add(Node* node)
{
	
	if(nullboids.size() != 0)
	{
		int index = nullboids.pop();
		
		boidList[index] = node;
		
	
		
	}
	else
	{

		boidList.push_back(node);
	}
}



void BucketIndex::initialize(int c, int r, Environment* Environment)
{

	index = new Bucket*[c * r];
	int k = 0;
	
	for(int j = 0; j < r; j++)
	{
		for(int i = 0; i < c; i++)
		{	
			index[k++] = new Bucket(i, j);
			
		}
	}


	extents[0] = 0;
	extents[1] = 0;
	extents[2] = c;
	extents[3] = r;
}



void BucketIndex::query(std::vector<Bucket*>* blist, float x0, 
							float y0, float x1, float y1)
{


	int c0 = int(x0);
	int r0 = int(y0);
	int c1 = int(x1);
	int r1 = int(y1);


	if(c0 < 0) c0 = 0;
	if(c1 > extents[2]) c1 = int(extents[2]);
	if(r0 < 0) r0 = 0;
	if(r1 > extents[3]) r1 = int(extents[3]);
	
	
	for(int i = c0; i < c1; i++)
	{
		for(int j = r0; j < r1; j++)
		{
		
			blist->push_back(index[i + int(extents[3])*j]);
		}
	}

}

Bucket* BucketIndex::query(float x, float y)
{


	int c = int(x);
	int r = int(y);

	
	if(x >= extents[0] && x <= extents[2] && 
				y >= extents[1] && y <= extents[3])
	{
		
		
		
		return index[c + int(extents[3])*r];
	}
	else
	{
		return (Bucket*) 0;
		
	}
		
	return (Bucket*) 0;
}










//Environment::Environment()
Environment::Environment(OggSound *snd) // RATKO
{


	geom = new Geometry();
	Color *color = new Color(0.75f, 0.75f, 0.75f);
	Shapes::plane(geom, color);


	tex = new Texture("data/underwater.jpg", IMAGE_JPEG);
	
	bubbleTex = new Texture("data/bubble.png", IMAGE_PNG);
	fishTex = new Texture("data/fish0.png", IMAGE_PNG);
	
	MoCap *mocap = new MoCap();
	mocap->sound = snd;  //set the sound data // RATKO
	mocap->read("data/mocap.txt");
	
	/* Create a bunch of boids positioned randomly */
	for(int i = 0; i < 100; i++)
	{
	
	
		Boid *boid = new Boid(fishTex);
		boid->translate(float(rrand(90)/100.0)+0.1, float(rrand(90)/100.0)+0.1, 0.0);
	
		/* need to fix this */
		boid->scale(0.001, 0.001, 0.001);
		
		boid->setId(i);
		attachChild(boid);
		
		
		boid->type = BOID_FISH;
		
		
		boid->sensorList = &sensorList;
	}
	
	
	Sensor* sensor;
	{
		/*
		sensor = new Sensor();
		sensor->mocap = mocap;
		sensor->sensorId = 0;
		attachChild(sensor);
		sensorList.push_back(sensor);
		*/
		sensor = new Sensor();
		sensor->mocap = mocap;
		sensor->sensorId = 0;
		attachChild(sensor);
		sensorList.push_back(sensor);
		
		
		sensor = new Sensor();
		sensor->mocap = mocap;
		sensor->sensorId = 1;
		attachChild(sensor);
		sensorList.push_back(sensor);
		
		sensor = new Sensor();
		sensor->mocap = mocap;
		sensor->sensorId = 2;
		attachChild(sensor);
		sensorList.push_back(sensor);
		
		/*
		sensor = new Sensor();
		sensor->mocap = mocap;
		sensor->sensorId = 3;
		attachChild(sensor);
		sensorList.push_back(sensor);
		/*
		sensor = new Sensor();
		sensor->mocap = mocap;
		sensor->sensorId = 5;
		attachChild(sensor);
		sensorList.push_back(sensor);
		*/
	}
	
	/*
	
	Creature *creature = new Creature();
	//creature->translate(0.5, 0.5, 0.0);
	creature->sensorList = &sensorList;
	
	attachChild(creature);
	
	*/
	
	bucketIndex.initialize(10, 10, this);
	
	
	initialized = false;
	
	
	
	Fl::add_timeout(0.0, dataTimer, mocap); 
}


BucketIndex* Environment::getBucketIndex()
{

	return &bucketIndex;
}


int Environment::cull()
{
	
	if(!initialized)
		initialize();

	float FRUSTUM[6][4];
	Frustum::get_frustum(FRUSTUM);


	if(Frustum::test_frustum(FRUSTUM, bbox.getBounds()) != -1)
	{
		draw();
	}
	else
		return -1;
	
	return 1;
	
}

void Environment::initialize()
{
		
	computeBBox();
	createBucketLines();
	
	
	initialized = true;
}

int emitt = 0;

void Environment::draw()
{



	if(tex && !tex->isLoaded())
	{
		
		tex->load();
	
	}
	
	
	if(geom && !geom->isReady())
	{
		
		geom->genDisplayList();
	
	}	
	
	
	
	if(geom)
	{
	
		

		if(tex)
		{
			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D, *(tex->getId()));
			
		}
	
	
			
		
		glCallList(*(geom->getDisplayList()));
		//glCallList(bucketList);
		
		
		if(tex)
		{
			glBindTexture(GL_TEXTURE_2D, 0);
			glDisable(GL_TEXTURE_2D);
		}
	
	}
	
	
	
	if(emitt >= 20)
	{
	
		Boid *boid;
		
		boid = new Boid(bubbleTex);
		boid->translate(0.5, 0.01 , 0.0);
		/* need to fix this */
		boid->scale(0.001, 0.001, 0.001);
		boid->setId(10);
		attachChild(boid);		
		boid->type = BOID_BUBBLE;	
		boid->sensorList = &sensorList;
	
	
	
		boid = new Boid(bubbleTex);
		boid->translate(0.8, 0.01 , 0.0);
		/* need to fix this */
		boid->scale(0.001, 0.001, 0.001);
		boid->setId(10);
		attachChild(boid);		
		boid->type = BOID_BUBBLE;	
		boid->sensorList = &sensorList;
		
		
		boid = new Boid(bubbleTex);
		boid->translate(0.2, 0.01 , 0.0);
		/* need to fix this */
		boid->scale(0.001, 0.001, 0.001);
		boid->setId(10);
		attachChild(boid);		
		boid->type = BOID_BUBBLE;	
		boid->sensorList = &sensorList;
	
	
		emitt = 0;
	}

	emitt++;
}


void Environment::createBucketLines()
{

	bucketList = glGenLists(1);

	glNewList(bucketList, GL_COMPILE);
	
	

	glColor3f(0.5, 0.5, 0.5);
	
	glBegin(GL_LINES);


	{
		int i;
		float j;
		for(j = 0; j < 1.0;)
		{
				
			glVertex3f(0.0, j, 0.0);
			glVertex3f(1.0, j, 0.0);
		
			j += 0.01;
			i++;
		}
	}
	glEnd();
	
	
	glBegin(GL_LINES);


	{
		float i;
		int j;
		for(i = 0; i < 1.0;)
		{
				
			glVertex3f(i, 0.0, 0.0);
			glVertex3f(i, 1.0, 0.0);
		
			i += 0.01;
			j++;
		}
	}
	
	glEnd();
	
	
	glEndList();

}


void Environment::computeBBox()
{

	bbox.set(0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 0.0f);
}




RTTI_NODE_TYPE Environment::getType() const
{

	return RTTI_USER0;

}





