
#ifndef Environment_H
#define Environment_H


#include "../../src-core/Node.h"
#include "../../src-core/Shapes.h"

#include "../../src-sound/OggSound.h" // RATKO added 
#include <math.h>  // RATKO


class Environment;

class Bucket
{

public:
	Bucket();
	Bucket(int, int);

	void remove(Node*);
	void add(Node*);
	int x();
	int y();
	
	std::vector<Node*>* getBoids();

private:
	
	int index[2];
	std::vector<Node*> boidList;
	NodeStack nullboids;
};


class BucketIndex
{

public:

	void initialize(int, int, Environment*);
	Bucket* query(float, float);
	void query(std::vector<Bucket*>*, float, float, float, float);
	
	float* getExtents();	

private:

	float extents[4];
	Bucket** index;


};



class Environment: public Node
{

public:
	//Environment();
	// RATKO
	Environment(OggSound*);  //added this constructor
	
	virtual int cull();
	virtual void draw();
	virtual RTTI_NODE_TYPE getType() const;
	
	virtual void computeBBox();
	
	virtual void doupdate() { }
	virtual void drawBBox() { }
	virtual void userEvent(UserEvent*) { }
	virtual void addTexture(Texture*) { }
	virtual Texture* getTexture() { }
	
	virtual BBox* getBBox() { }
	virtual void addGeometry(Geometry*) { }
	virtual Geometry* getGeometry() { }
	virtual void print() { }


	void initialize();
	
	BucketIndex* getBucketIndex();
	
	
private:

	

	bool initialized;
	
	void createBucketLines();

	Geometry *geom;
	Texture *tex;
	BBox bbox;

	Texture *fishTex;
	Texture *bubbleTex;	

	GLuint bucketList;
	BucketIndex bucketIndex;
	
	
	
	std::vector<Node*> obstacleList;
	std::vector<Node*> sensorList;
	
	
};





#endif

