


#include "TileConfig.h"


int TileConfig::readDisplayCfg(char* filename)
{

	FILE* fin = fopen(filename, "r");
	if(fin == NULL)
	{
		return -1;	
	}

	char linebuffer[32];
	
	fscanf(fin, "%s", linebuffer);
	if(strcmp(linebuffer, "Cluster"))
	{
		return -1;	
	}
	
	cluster = new Cluster();
		
	fscanf(fin, "%s", linebuffer);
	if(strcmp(linebuffer, "TileMap"))
	{
		return -1;
	}
	fscanf(fin, "%d %d\n", &cluster->tilemap[0], &cluster->tilemap[1]);
	
	fscanf(fin, "%s", linebuffer);
	if(strcmp(linebuffer, "Mullions"))
	{
		return -1;	
	}
	
	fscanf(fin, "%f %f\n", &cluster->mullions[0], &cluster->mullions[1]);
#ifdef DEBUG_TileConfig_PARSER	
	printf("mullions: %f %f\n", cluster->mullions[0], 
							cluster->mullions[1]);
#endif
	fscanf(fin, "%s", linebuffer);
	
	if(strcmp(linebuffer, "Nodes"))
	{
		return -1;;
	}

	int nOfNodes;
	
	fscanf(fin, "%d\n", &nOfNodes);
#ifdef DEBUG_TileConfig_PARSER	
	printf("# of nodes = %i\n", nOfNodes);	
#endif

	for(int n = 0; n < nOfNodes; n++)	
	{
		fscanf(fin, "%s", linebuffer);
		if(strcmp(linebuffer, "Node"))
		{
			return -1;
		}
		
		ClusterNode *cNode = new ClusterNode();

		fscanf(fin, "%s", linebuffer);
		if(strcmp(linebuffer, "Hostname"))
		{
			return -1;
		}	
		
		fscanf(fin, "%s", cNode->hostname);	
#ifdef DEBUG_TileConfig_PARSER
		printf("node %i\n", n);
#endif

#ifdef DEBUG_TileConfig_PARSER
		printf("Hostname: %s\n", cNode->hostname);
#endif


		fscanf(fin, "%s", linebuffer);
		if(strcmp(linebuffer, "Windows"))
		{
			return -1;
		}
		
		int nOfWindows;
		fscanf(fin, "%d\n", &nOfWindows);
#ifdef DEBUG_TileConfig_PARSER
		printf("# of windows = %i\n", nOfWindows);
#endif

		for(int w = 0; w < nOfWindows; w++)
		{

			WindowEntity *we = new WindowEntity();
			
			fscanf(fin, "%s", linebuffer);
			if(strcmp(linebuffer, "Map"))
			{
				return -1;
			}
			fscanf(fin, "%d %d\n", &we->mapping[0], &we->mapping[1]);
			
			fscanf(fin, "%s", linebuffer);
			if(strcmp(linebuffer, "Location"))
			{
				return -1;
			}
			
			fscanf(fin, "%d %d %d %d\n",
						&we->location[0], &we->location[1], 
						&we->location[2], &we->location[3]);
#ifdef DEBUG_TileConfig_PARSER
			printf("%i at %i %i %i %i\n", w, we->location[0],
				we->location[1], we->location[2], we->location[3]);
#endif

			fscanf(fin, "%s", linebuffer);
			if(strcmp(linebuffer, "Cameras"))
			{
				return -1;
			}
			
			int nOfCams;
			fscanf(fin, "%d\n", &nOfCams);
#ifdef DEBUG_TileConfig_PARSER
			printf("# of cameras = %i\n", nOfCams);
#endif			
			for(int c = 0; c < nOfCams; c++)
			{

				CameraEntity *ce = new CameraEntity();
				
				fscanf(fin, "%s", linebuffer);
				if(strcmp(linebuffer, "Rectangle"))
				{
					return -1;
				}

				fscanf(fin, "%f %f %f %f\n",
						&ce->rectangle[0], &ce->rectangle[1],
							&ce->rectangle[2], &ce->rectangle[3]);
#ifdef DEBUG_TileConfig_PARSER
				printf("rectangle: %f %f %f %f\n", 
						ce->rectangle[0], ce->rectangle[1], ce->rectangle[2],
								ce->rectangle[3]);

#endif

				fscanf(fin, "%s", linebuffer);
				if(strcmp(linebuffer, "Offset"))
				{
					return -1;
				}
				
				fscanf(fin, "%f %f\n", &ce->offset[0], &ce->offset[1]);
#ifdef DEBUG_TileConfig_PARSER
				printf("offset: %f %f\n", ce->offset[0], ce->offset[1]);
#endif


				we->cList.push_back(ce);
				
			}	

			cNode->wList.push_back(we);
	
		}

		cluster->nList.push_back(cNode);


	}

	fclose(fin);
	
	cfgNode();
}


void TileConfig::cfgNode()
{
	thisNode = NULL;
	
#ifdef WIN32
	thisNode = cluster->nList.front();
#else	
	char hostname[32];
	gethostname(hostname, 32);

	printf("This node's hostname: %s\n", hostname);


	for(int i = 0; i < cluster->nList.size(); i++)
	{
		ClusterNode* n = cluster->nList[i];
		if(!strcmp(n->hostname, hostname))
		{
			thisNode = n;
			break;
		}

	}
#endif

	if(thisNode == NULL)
	{
		printf("No information for this node!\n");
		exit(0);
	}
	else
	{
		printf("cfg for %s identified!\n", hostname);
	}
	
}




ClusterNode* TileConfig::getThisNodeCfg()
{

	return thisNode;
}
