#include "Geometry.h"

Geometry::Geometry()
{
	ready = false; 
	
	glVertices = (std::vector<Point*>*) 0;
	glTCoords = (std::vector<Point*>*) 0;

	pointSize = 1.0f;
	lineSize = 1.0f;
	depth = 0.0f;
}


Geometry::Geometry(const char* gname, Color* c, int type)
{

	name = gname;
	geomType = type;
	color = c;
	
	ready = false; 
	
	glVertices = (std::vector<Point*>*) 0;
	glTCoords = (std::vector<Point*>*) 0;

	pointSize = 1.0f;
	lineSize = 1.0f;
	depth = 0.0f;
}

void Geometry::set(const char* gname, Color* c, int type)
{
	name = gname;
	geomType = type;
	color = c;

}


void Geometry::setVertices(std::vector<Point*>* v)
{

	glVertices = v;

}

void Geometry::setPointSize(float s)
{
	pointSize = s;
}

void Geometry::setLineSize(float s)
{
	lineSize = s;
}

void Geometry::setTCoords(std::vector<Point*>* tc)
{

	glTCoords = tc;
}


GLuint* Geometry::getDisplayList()
{
	return &dList;
}

void Geometry::genDisplayList()
{
	
	if(!glVertices && !glTCoords)
		return;
		
		
	dList = glGenLists(1);

	glNewList(dList, GL_COMPILE);
	

	glPointSize(pointSize);
	glLineWidth(lineSize);
	
	glBegin(geomType);
	
	glColor3f(color->get()[0], color->get()[1], color->get()[2]);
	
	
	for(int i = 0; i < glVertices->size(); i++)
	{
		
		if(glTCoords)
			glTexCoord2f((*glTCoords)[i]->get()[0], (*glTCoords)[i]->get()[1]); 
		
		glVertex3f((*glVertices)[i]->get()[0], (*glVertices)[i]->get()[1], depth);
		
	}
	
	glPointSize(1.0f);
	glLineWidth(1.0f);

	
	glEnd();
	
	
	glEndList();
	
	
	
	ready = true;
}

void Geometry::setDepth(float d)
{
	depth = d;
}

bool Geometry::isReady() 
{
	return ready;
}


const char* Geometry::getName()
{
	return name;
}



