

#ifndef Node_H
#define Node_H

#include <vector>
#include <stdlib.h>


#include "Math.h"
#include "BBox.h"
#include "Geometry.h"
#include "Texture.h"
#include "UserEvent.h"
#include "Frustum.h"
#include "FreeType.h"


#include <list>


#define DRAWBBOXES




enum RTTI_NODE_TYPE
{
	RTTI_SCENE,
	RTTI_PIVOT,
	RTTI_PARTICLE,
	RTTI_TERRAIN,
	RTTI_SPECIAL,
	RTTI_USER0,
	RTTI_USER1,
	RTTI_USER2,
	RTTI_USER3,
	RTTI_USER4,
	RTTI_USER5,
	RTTI_USER6,
	RTTI_USER7,
	RTTI_USER8,
	RTTI_USER9
};


enum CULL_RESULT
{

	NODE_OUTSIDE,
	NODE_INSIDE,
	NODE_INTERSECTS

};





class NodeStack
{

public:
	
	NodeStack();
	
	void push(int);
	int pop();
	int size();

private:

	
	std::list<int> stack;

};


class Node
{

public: static int darray[9][2];


public:
	Node();
	~Node();
	

	void attachChild(Node*);
	void setParent(Node*);
	std::vector<Node*>* getChildList();
	Node* getParent();
	float* glTransform();
	float* glWorldMatrix();
	float* getViewPoint();
	const char* getName();
	void setName(const char*);
	void setUpdateFunc(void (*) (Node*));
	void update();
	virtual void translate(float, float, float);
	virtual void scale(float, float, float);
	virtual void rotate(float, float, float, float);
	
	float* parallelScale();

	
	virtual int cull()=0;
	virtual void draw()=0;
	virtual void doupdate()=0;
	virtual void drawBBox()=0;
	virtual RTTI_NODE_TYPE getType() const = 0;

	virtual void userEvent(UserEvent*)=0;
	
	virtual void addTexture(Texture*)=0;
	virtual Texture* getTexture()=0;
		
	virtual void computeBBox()=0;
	virtual BBox* getBBox()=0;
	
	virtual void addGeometry(Geometry*)=0;
	virtual Geometry* getGeometry()=0;

	virtual void print()=0;
	
	
	bool isAlive();
	void die();
	
	void appendNull(int);
	
	
private:
	
	bool alive;
	
	// stack of unused child pointers of a node
	NodeStack nullchildren;
	
	std::vector<Node*> children;
	Node* parent;

	float viewpoint[2];
	
	float transformmat[16];
	float worldmat[16];
	
	const char* name;
	
	
	void (*updateptr) (Node*);
	
	
	
	float pscale;
	

	
};



#endif

