	
#include "Window.h"


void timer(void *pData) {
	
	if( pData != NULL ) {
	
		Window *w = reinterpret_cast<Window*>(pData);
	
		w->fps = w->frames;
		w->frames = 0;
	}

	
	Fl::repeat_timeout(1.0, timer, pData);
}



Window::Window(int id, int x, int y, int w, int h, char* t): Fl_Gl_Window(x, y, w, h, t)
{

	mode(FL_RGB | FL_ALPHA | FL_DEPTH | FL_DOUBLE | FL_ACCUM);
	
	resizable(this);

	
	windowId = id;
	
	height = h;
	width = w;
	


	pscale = 1.0f;	
	
	frames = 0;
	fps = 0.0f;
	
	
	origin[0] = x;
	origin[1] = y;
	
	
	
	drawcallback = (void (*) (Window*)) 0;
	
	
	Fl::add_timeout(0.0, timer, this);
}

void Window::setOffset(float x0, float y0)
{
	offset[0] = x0;
	offset[1] = y0;

}

void Window::initGL() 
{
	valid(1);
	
	glClearColor(0.25f, 0.25f, 0.25f, 1.0f);
	
	/*
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);
	*/
	
    glEnable(GL_SCISSOR_TEST);
    
    /*
    // lighting
    glEnable(GL_LIGHTING);
    
	glEnable(GL_LIGHT0);
	float lightColor[4]= {1.0, 1.0, 1.0, 1.0};
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightColor);
	
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
    */
    
        	
	render.getCamera()->ortho2D(0, 0, width, height, offset);
	our_font->init("fonts/ChalkboardBold.ttf", 12);
	pscale = render.getSG()->getRootNode()->glTransform()[0];

}

void Window::resize(int x, int y, int w, int h)
{

	render.getCamera()->reshape(x, y, w, h);

	
}

void Window::idle()
{

	if(windowId != 1)
		return;
	

	if(valid())
	{
		render.idleFrame();
	
		
		
	}
}	

void Window::setFont(freetype::font_data* f)
{

	our_font = f;
}

void Window::draw()
{
	
	
	if(!valid()) 
	{
		initGL();
		
	}
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	
	render.frame();

		
	float scale;
	Point p0(0.0f, 0.0f);
	Point p1(1.0f, 1.0f);
	Point p0s;
	Point p1s;
	render.getCamera()->unproject(&p0s, &p0, 
				render.getSG()->getRootNode()->glTransform());
	render.getCamera()->unproject(&p1s, &p1,
				render.getSG()->getRootNode()->glTransform());
	
	scale = p1s.get()[0] - p0s.get()[0];
	
	glColor4f(1.0, 1.0, 1.0, 0.5);
	freetype::print(1, *our_font, 10, 10,
		 "%3.1f fps", 
		 fps );
	
		
	
	
	if(drawcallback)
		drawcallback(this);
	
	
	frames++;
}


int Window::handle(int evt)
{

	switch ( evt ) 
	{
	
		case FL_PUSH:
			if(Fl::event_button() == 3)
			{
				int mposx = Fl::event_x();
				int mposy = Fl::event_y();

				
				
				
				UserEvent *e = new UserEvent(MOUSE_CLICK);
				float* data = e->getEventData();
				data[0] = (float) mposx;
				data[1] = (float) mposy;
				data[2] = pscale;
				
				
				
				render.getSG()->event(e);
				
				
				MOUSE[0] = mposx;
				MOUSE[1] = mposy;
				
			}
		
			return 1;
		
		
		case FL_DRAG:
			/* Translates root instead of pivot node
			if(Fl::event_key() == 's')
			{
				
				int dposx = Fl::event_x();
				int dposy = Fl::event_y();
				
				
				int move_vec[2];
				move_vec[0] = dposx - MOUSE[0];
				move_vec[1] = dposy - MOUSE[1];
				
		
				render.getSG()->getRootNode()->translate(
							move_vec[0]/pscale, -move_vec[1]/pscale);
				
				MOUSE[0] = dposx;
				MOUSE[1] = dposy;
			}
			*/
			
			if(Fl::event_button() == 1)
			{
			
				int dposx = Fl::event_x();
				int dposy = Fl::event_y();
				
				
				int move_vec[2];
				move_vec[0] = dposx - MOUSE[0];
				move_vec[1] = dposy - MOUSE[1];
				
		
				render.getSG()->getPivotNode()->translate(
						float(move_vec[0])/1000.0f, float(-move_vec[1])/1000.0f, 0.0);
				
				
				MOUSE[0] = dposx;
				MOUSE[1] = dposy;
			
			}
			
			
			if(Fl::event_key() == 'z')
			{
			
				int dposx = Fl::event_x();
				int dposy = Fl::event_y();
				
				
				int move_vec[2];
				move_vec[0] = dposx - MOUSE[0];
				move_vec[1] = dposy - MOUSE[1];
				
		
				render.getSG()->getPivotNode()->rotate(
							0.0f, 0.0f, 1.0f, move_vec[1]/pscale);
				
				
				
				MOUSE[0] = dposx;
				MOUSE[1] = dposy;
			
			}
			
			if(Fl::event_key() == 'x')
			{
			
				int dposx = Fl::event_x();
				int dposy = Fl::event_y();
				
				
				int move_vec[2];
				move_vec[0] = dposx - MOUSE[0];
				move_vec[1] = dposy - MOUSE[1];
				
		
				render.getSG()->getPivotNode()->rotate(
							1.0f, 0.0f, 0.0f, move_vec[1]/pscale);
				
				
				
				MOUSE[0] = dposx;
				MOUSE[1] = dposy;
			
			}
			
			if(Fl::event_key() == 'c')
			{
			
				int dposx = Fl::event_x();
				int dposy = Fl::event_y();
				
				
				int move_vec[2];
				move_vec[0] = dposx - MOUSE[0];
				move_vec[1] = dposy - MOUSE[1];
				
		
				render.getSG()->getPivotNode()->rotate(
							0.0f, 1.0f, 0.0f, move_vec[0]/pscale);
				
				
				
				MOUSE[0] = dposx;
				MOUSE[1] = dposy;
			
			}
			
			return 1;
		
		
			
		case FL_MOUSEWHEEL:
			
			{
			
				int value = Fl::event_dy();
			
			
			
				if(value < 0) 
				{
					render.getSG()->getRootNode()->scale(1.25, 1.25, 1);
					pscale *= 1.25;	
				}
				if(value > 0)
				{
					render.getSG()->getRootNode()->scale(0.8, 0.8, 1); 
					pscale /= 1.25;
				}
			}
		
			return 1;
		
		
		case FL_MOVE:
			{
				int mposx = Fl::event_x();
				int mposy = Fl::event_y();
				
				MOUSE[0] = mposx;
				MOUSE[1] = mposy;
				
			
			
		
			}
			
			return 1;
	
		case FL_KEYBOARD:
			switch(Fl::event_key())
			{
				
				
				case 'Q':
				case 'q':
					hide();
					break;
				
				case 32:
					int mposx = MOUSE[0];
					int mposy = MOUSE[1];
	
					
					
					
					UserEvent *e = new UserEvent(MOUSE_CLICK);
					float* data = e->getEventData();
					data[0] = (float) mposx;
					data[1] = (float) mposy;
					data[2] = pscale;
					
					
					
					render.getSG()->event(e);
					
					break;				
			}
			
			
			return 1;
		
	}
	
}

int Window::getWindowId() {
	return windowId;
}


Render* Window::getRender()
{
	return &render;
}

void Window::setDrawCallback(void (*ptr) (Window*))
{

	drawcallback = ptr;

}



