
#include "ImageJPEG.h"



ImageJPEG::ImageJPEG()
{
	buffer = (unsigned char*) 0;
	w = 0;
	h = 0;
	c = 0;
	
}

ImageJPEG::~ImageJPEG() 
{
	if(buffer)
		freeBuffer();
		
}


struct my_error_mgr 
{
	struct jpeg_error_mgr pub;
  	jmp_buf setjmp_buffer;
};

typedef struct my_error_mgr * my_error_ptr;

void my_error_exit (j_common_ptr cinfo)
{
  	my_error_ptr myerr = (my_error_ptr) cinfo->err;
  	(*cinfo->err->output_message) (cinfo);
  	longjmp(myerr->setjmp_buffer, 1);
}


int ImageJPEG::read(char* filename)
{
	struct jpeg_decompress_struct cinfo;

	struct my_error_mgr jerr;

	FILE* infile;
	JSAMPARRAY linebuffer;
	int row_stride;

	if((infile = fopen(filename, "rb")) == NULL)
	{
		printf("can't open %s\n", filename);
		return 0;
	}
	else 
	{
		//printf("Opened %s\n", filename);
	}

	cinfo.err = jpeg_std_error(&jerr.pub);
	jerr.pub.error_exit = my_error_exit;

	if(setjmp(jerr.setjmp_buffer))
	{
		jpeg_destroy_decompress(&cinfo);
		fclose(infile);
		return 0;
	}



	jpeg_create_decompress(&cinfo);
	jpeg_stdio_src(&cinfo, infile);
	jpeg_read_header(&cinfo, TRUE);
	jpeg_start_decompress(&cinfo);
	
	row_stride = cinfo.output_width * cinfo.output_components;

	linebuffer = (*cinfo.mem->alloc_sarray)
		((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);


	w = cinfo.output_width;
	h = cinfo.output_height;
	c = cinfo.output_components;
	
	freeBuffer();
	
	
	buffer = new unsigned char[w * h * c];
	memset(buffer, 0, c * w * h );

		
#ifdef DEBUG_JPEG
	printf("width = %i height = %i components = %i\n",
	cinfo.output_width, cinfo.output_height, cinfo.output_components);
#endif

	//this makes an upside-down buffer
	//int index = 0;
	int index = (h - 1) * sizeof(unsigned char) * row_stride;

	while(cinfo.output_scanline < cinfo.output_height)
	{
#ifdef DEBUG_JPEG
		printf("%i ", cinfo.output_scanline);
#endif
		jpeg_read_scanlines(&cinfo, linebuffer, 1);
	
		memcpy(buffer + index, linebuffer[0], sizeof(unsigned char)*row_stride);
		//this makes an upside-down buffer
		//index = index + sizeof(unsigned char)*row_stride;
		index = index - sizeof(unsigned char)*row_stride;	
	}	

	
	jpeg_finish_decompress(&cinfo);
	jpeg_destroy_decompress(&cinfo);

	fclose(infile);
	
	
	sourceFile = new char[ strlen(filename) + 1];
    strcpy(sourceFile, filename);

	return 1;
}


int ImageJPEG::write(char* filename, int quality)
{

	if(!buffer)
		return 0;
	
	JSAMPLE* image_buffer = buffer;
  	int image_height = getHeight();
  	int image_width = getWidth();
  	int image_component = getComponents();
  	
  	struct jpeg_compress_struct cinfo;
  	struct jpeg_error_mgr jerr;
  

  	FILE * outfile;	/* target file */
  	JSAMPROW row_pointer[1]; /* pointer to JSAMPLE row[s] */
  	int row_stride;	/* physical row width in image buffer */

  	cinfo.err = jpeg_std_error(&jerr);
  	/* Now we can initialize the JPEG compression object. */
  	jpeg_create_compress(&cinfo);

  	if ((outfile = fopen(filename, "wb")) == NULL) {
    	fprintf(stderr, "can't open %s\n", filename);
    	return 0;
  	}
  
  	jpeg_stdio_dest(&cinfo, outfile);

  	cinfo.image_width = image_width; /* image width and height, in pixels */
  	cinfo.image_height = image_height;
  	cinfo.input_components = image_component; /* # of color components per pixel */
  
  	cinfo.in_color_space = JCS_RGB; /* colorspace of input image */
 	jpeg_set_defaults(&cinfo);
  	jpeg_set_quality(&cinfo, quality, TRUE /* limit to baseline-JPEG values */);

  	/* Step 4: Start compressor */

  	/* TRUE ensures that we will write a complete interchange-JPEG file.
   	* Pass TRUE unless you are very sure of what you're doing.
   	*/
 	jpeg_start_compress(&cinfo, TRUE);
#ifdef DEBUG_JPEG
   	printf("start compress: %d * %d\n", cinfo.image_width, cinfo.image_height);
#endif 
  	row_stride = image_width * image_component;	/* JSAMPLEs per row in image_buffer */

  	while (cinfo.next_scanline < cinfo.image_height) {
     	row_pointer[0] = & image_buffer[cinfo.next_scanline * row_stride];
    	(void) jpeg_write_scanlines(&cinfo, row_pointer, 1); 
  	}
#ifdef DEBUG_JPEG
  	printf("\n finished compressing!\n");
#endif
  	/* Step 6: Finish compression */

  	jpeg_finish_compress(&cinfo);
  	fclose(outfile);
  	jpeg_destroy_compress(&cinfo);


	return 1;
}

void ImageJPEG::freeBuffer()
{
	
	delete [] buffer;
	buffer = (unsigned char*) 0;
#ifdef DEBUG_JPEG	
	printf("Freed pixel buffer!\n");
#endif
}

unsigned char* ImageJPEG::getPixelBuffer()
{

	return buffer;

}


int ImageJPEG::getWidth()
{
	return w;
}

int ImageJPEG::getHeight()
{
	return h;
}


int ImageJPEG::getComponents() 
{

	return c;
}

char* ImageJPEG::getSourceFile()
{

	return sourceFile;
}

int ImageJPEG::getImageType()
{

	return IMAGE_JPEG;
	
}


