

#include "ImagePNG.h"



ImagePNG::ImagePNG()
{
	buffer = (unsigned char*) 0;
	w = 0;
	h = 0;
	c = 0;
	
}

ImagePNG::~ImagePNG() 
{
	if(buffer)
		freeBuffer();
		
}

int ImagePNG::read(char* filename)
{

	if( filename == NULL )
        return -1;
        
    if( !strstr(filename,".PNG") && !strstr(filename,".png") )
    {
        return -1;
    }


    png_structp png_ptr;
    png_ptr = png_create_read_struct( PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
    if( !png_ptr )
        return -1;

    png_infop info_ptr;
    info_ptr = png_create_info_struct(png_ptr);
    if( !info_ptr )
    {
        png_destroy_read_struct( &png_ptr, NULL, NULL );
        return -1;
    }

    if( setjmp( png_jmpbuf( png_ptr )))
    {
        png_destroy_read_struct( &png_ptr, &info_ptr, NULL );
        return -1;
    }

    FILE* fp = fopen( filename, "rb" );
    if(!fp)
    {
        fprintf(stderr,"Can't open file %s\n", filename);
        return -1;
    }

    png_init_io(png_ptr,fp);
    png_read_png( png_ptr, info_ptr,
        PNG_TRANSFORM_STRIP_16 | PNG_TRANSFORM_PACKING, NULL );

    w = (int) png_get_image_width(png_ptr,info_ptr);
    h = (int) png_get_image_height(png_ptr,info_ptr);

    int channelCount;
        
    /*               
    resH = png_get_x_pixels_per_meter(png_ptr,info_ptr);
    resV = png_get_y_pixels_per_meter(png_ptr,info_ptr);
	*/
	
    switch( png_get_color_type(png_ptr,info_ptr) )
    {
    case PNG_COLOR_TYPE_GRAY:
      
        channelCount = 1;
        break;
    case PNG_COLOR_TYPE_RGB:
      
        channelCount = 3;
        break;
    case PNG_COLOR_TYPE_RGB_ALPHA:
       
        channelCount = 4;
        break;
    default:
        fprintf(stderr,"Unsupported color type!\n");
        png_destroy_read_struct( &png_ptr, &info_ptr, NULL );
        fclose(fp);
        return -1;
    };
 
    //only support 8 bit
    if( info_ptr->bit_depth != 8 )
    {
       fprintf(stderr,"Unsupported bit depth!\n");
       png_destroy_read_struct( &png_ptr, &info_ptr, NULL);
       fclose(fp);
       return -1;
    }

    png_bytep *b = png_get_rows( png_ptr, info_ptr );

    if( !b )
    {
        fprintf(stderr,"Can't get pixels from PNG\n");
        png_destroy_read_struct( &png_ptr, &info_ptr, NULL);
        fclose(fp);
        return -1;
    }

	c = channelCount;
	
	freeBuffer();
	
    buffer = new unsigned char[ c * w * h ];
    memset(buffer, 0, c * w * h );
    
    for( int y = 0; y < h; y++)
        for(int x = 0; x < w; x++)
            for( int i = 0; i < channelCount; i++)
                buffer[ y * w * channelCount + x * channelCount + i]
                    = (char) b[h - y - 1][x * channelCount + i];

    png_destroy_read_struct( &png_ptr, &info_ptr, NULL );
    fclose(fp);

    sourceFile = new char[ strlen(filename) + 1];
    strcpy( sourceFile, filename );

    return 1;

}

int ImagePNG::write(char* filename, int quality)
{


}

void ImagePNG::freeBuffer()
{
	
	delete [] buffer;
	buffer = (unsigned char*) 0;
#ifdef DEBUG_JPEG	
	printf("Freed pixel buffer!\n");
#endif
}

unsigned char* ImagePNG::getPixelBuffer()
{

	return buffer;

}


int ImagePNG::getWidth()
{
	return w;
}

int ImagePNG::getHeight()
{
	return h;
}


int ImagePNG::getComponents() 
{

	return c;
}

char* ImagePNG::getSourceFile()
{
	return sourceFile;
}

int ImagePNG::getImageType()
{

	return IMAGE_PNG;
}


