
#include "Vector.h"

#define max(a,b) ((a>b)?a:b)
const float DEG2RAD = 3.14159/180;


Vec3::Vec3(float x, float y, float z)
{
	this->x = x;
	this->y = y;
	this->z = z;
}


Vec3::Vec3(const Vec3 &v)
{
	x = v.x;
	y = v.y;
	z = v.z;
}


Vec3::Vec3()
{
	x = 0;
	y = 0;
	z = 0;
}


Vec3 Vec3::operator+(const Vec3 &v)
{

	Vec3 res;

	res.x = x + v.x;
	res.y = y + v.y;
	res.z = z + v.z;

	return(res);
}

Vec3 Vec3::operator-(const Vec3 &v) {

	Vec3 res;

	res.x = x - v.x;
	res.y = y - v.y;
	res.z = z - v.z;

	return(res);
}

Vec3 Vec3::operator-(void)
{

	Vec3 res;

	res.x = -x;
	res.y = -y;
	res.z = -z;

	return(res);
}


Vec3 Vec3::operator*(float t)
{

	Vec3 res;

	res.x = x * t;
	res.y = y * t;
	res.z = z * t;

	return (res);
}

Vec3 Vec3::operator/(float t) 
{

	Vec3 res;

	res.x = x / t;
	res.y = y / t;
	res.z = z / t;

	return (res);
}


float Vec3::length()
{

	return((float)sqrt(x*x + y*y + z*z));
}


void Vec3::normalize()
{

	float len;

	len = length();
	if (len) {
		x /= len;
		y /= len;
		z /= len;
	}
}
	

void Vec3::copy(const Vec3 &v)
{

	x = v.x;
	y = v.y;
	z = v.z;
}


void Vec3::set(float x, float y, float z) 
{

	this->x = x;
	this->y = y;
	this->z = z;
}


	
void Vec3::limit(float lim)
{
   	float m, f;
	   
	m = max(fabs(x), fabs(y));
	m = max(m, fabs(z));
	   
	if(m <= lim) return;
	   
	f = lim/m;
	
	this->x *= f;
	this->y *= f;
	this->z *= f;
}


void Vec3::rshift(int n)
{

	this->x = (int)this->x >> n;
   	this->y = (int)this->y >> n;
   	this->z = (int)this->z >> n;

}

void Vec3::lshift(int n)
{

	this->x = (int)this->x << n;
   	this->y = (int)this->y << n;
   	this->z = (int)this->z << n;

}


bool Vec3::isnonzero()
{

	if(this->x == 0 & this->y == 0 && this->z == 0)
		return false;
	
	return true;

}

void Vec3::rotate(float alpha)
{

	alpha *= DEG2RAD;
	
	
	
	float co = cos(alpha);
	float si = sin(alpha);
	
	this->x = co * this->x + si * this->y;
	this->y = - si * this->x + co * this->y;
}

/* static methods */




bool Vec3::Equal(const Vec3& a, const Vec3& b)
{
	if(a.x != b.x || a.y != b.y || a.z != b.z)
		return false;
	
	return true;
}



float Vec3::DotProduct(const Vec3& a, const Vec3& b)
{
	return a.x * b.x + a.y * b.y + a.z * b.z;
}



Vec3 Vec3::CrossProduct(const Vec3& a, const Vec3& b)
{
	return Vec3(
		(a.y * b.z) - (a.z * b.y),
		(a.z * b.x) - (a.x * b.z),
		(a.x * b.y) - (a.y * b.x));
}


float Vec3::EuclDistance(const Vec3& a, const Vec3& b)
{
	Vec3 res;
	res.x = a.x - b.x;
	res.y = a.y - b.y;
	res.z = a.z - b.z;
	
	return sqrt( pow(res.x, 2) + pow(res.y, 2) + pow(res.z, 2) );
}


float Vec3::MinCompDist(const Vec3& a, const Vec3& b)
{
	float dx, dy, dz, dm;
   
  	dx = a.x - b.x;
   	dy = a.y - b.y;
   	dz = a.z - b.z;
   
   	dm = max(fabs(dx), fabs(dy));
   	dm = max(dm, fabs(dz));
   
   	return dm;
}



float Vec3::AngleBetween(const Vec3& a, const Vec3& b)
{
	
	
	return acos( Vec3::DotProduct(a, b) / 
		( ((Vec3&) a).length() * ((Vec3&) b).length() ) );
}

	
	
