
#include "OggSound.h"


OggSound::OggSound()
{

}

OggSound::~OggSound()
{
	
	cleanUp();

}

void OggSound::initAL(int* argc, char** argv)
{


	 // Initialize the OpenAL library
    alutInit(argc, argv);
}

void OggSound::exitAL()
{


	 // Clean up the OpenAL library
    alutExit();
}


void OggSound::cleanUp()
{

	// Clean up sound buffer and source
    alDeleteBuffers(1, &bufferID);
    alDeleteSources(1, &sourceID);
}


// This function loads a .ogg file into a memory buffer and returns
// the format and frequency.
void OggSound::loadOGG(char *fileName, std::vector<char> &buffer, ALenum &format, ALsizei &freq)
{
    int endian = 0;                         // 0 for Little-Endian, 1 for Big-Endian
    int bitStream;
    long bytes;
    char array[BUFFER_SIZE];                // Local fixed size array
    FILE *f;

    // Open for binary reading
    f = fopen(fileName, "rb");

    if (f == NULL)
        {
        std::cerr << "Cannot open " << fileName << " for reading..." << std::endl;
        exit(-1);
        }
    // end if

    vorbis_info *pInfo;
    OggVorbis_File oggFile;

    // Try opening the given file
    if (ov_open(f, &oggFile, NULL, 0) != 0)
        {
        std::cerr << "Error opening " << fileName << " for decoding..." << std::endl;
        exit(-1);
        }
    // end if

    // Get some information about the OGG file
    pInfo = ov_info(&oggFile, -1);
    
    // remember the number of samples in a file (used in syncronization with graphics)
	numSamples = ov_pcm_total(&oggFile, -1);  // RATKO

    // Check the number of channels... always use 16-bit samples
    if (pInfo->channels == 1)
        format = AL_FORMAT_MONO16;
    else
        format = AL_FORMAT_STEREO16;
    // end if

    // The frequency of the sampling rate
    freq = pInfo->rate;

    // Keep reading until all is read
    do
        {
        // Read up to a buffer's worth of decoded sound data
        bytes = ov_read(&oggFile, array, BUFFER_SIZE, endian, 2, 1, &bitStream);

        if (bytes < 0)
            {
            ov_clear(&oggFile);
            std::cerr << "Error decoding " << fileName << "..." << std::endl;
            exit(-1);
            }
        // end if

        // Append to end of buffer
        buffer.insert(buffer.end(), array, array + bytes);
        }
    while (bytes > 0);

    // Clean up!
    ov_clear(&oggFile);
}


void OggSound::load(char *filename)
{


	// Create sound buffer and source
    alGenBuffers(1, &bufferID);
    alGenSources(1, &sourceID);

    // Set the source and listener to the same location
    alListener3f(AL_POSITION, 0.0f, 0.0f, 0.0f);
    alSource3f(sourceID, AL_POSITION, 0.0f, 0.0f, 0.0f);

    // Load the OGG file into memory
    loadOGG(filename, bufferData, format, freq);

    // Upload sound data to buffer
    alBufferData(bufferID, format, &bufferData[0], 
    			static_cast<ALsizei>(bufferData.size()), freq);

    // Attach sound buffer to source
    alSourcei(sourceID, AL_BUFFER, bufferID);

   

}

void OggSound::play()
{
	 // Finally, play the sound!!!
    alSourcePlay(sourceID);

}


int OggSound::getState()
{

	alGetSourcei(sourceID, AL_SOURCE_STATE, &state);

	return state;
}

// RATKO
float OggSound::getPlaybackOffset()
{
	ALfloat timeOffset;
	alGetSourcef(sourceID, AL_SEC_OFFSET, &timeOffset);
	//printf("\nTotal Time: %f\n", timeOffset);
	alGetSourcef(sourceID, AL_SAMPLE_OFFSET, &sampleOffset);
	return sampleOffset/numSamples;
}

